@@ -330,10 +330,10 @@ public class ConfigurationProperties {
     // --- JSON/OLOX2 ---
 
     // TODO Create properties for this value ! Why this strange default value ?
-    public static final long START_DATE_MINUS_DEFAULT = 311040000L;// 36000*24*360
+    public static final long START_DATE_MINUS_DEFAULT = 311040000L;// 36*1000*24*360 = 3.6 days
 
     // TODO Create properties for this value ! Why this strange default value ?
-    public static final long END_DATE_PLUS_DEFAULT = 3110400L;// 360*24*360
+    public static final long END_DATE_PLUS_DEFAULT = 3110400L;// 360*24*360 = 51.84 minutes
 
     public static final String JSON_SERVLET_ALIAS_PROPERTY = "com.openexchange.usm.json.alias";
 
