@@ -49,6 +49,7 @@
 
 package com.openexchange.usm.contenttypes.calendar.impl;
 
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.HashMap;
@@ -212,38 +213,70 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
         Map<String, String> parameters = new HashMap<String, String>();
 
         parameters.put(CommonConstants.COLUMNS, createColumnsParameter(requestedFields));
-        addTimeLimitParameters(parameters, session);
+
         String folder = seriesObject != null ? seriesObject.getParentFolderID() : null;
         if (folder != null)
             parameters.put(CommonConstants.FOLDER, folder);
         parameters.put(CommonConstants.RECURRENCE_MASTER, Boolean.FALSE.toString());
         addObjectsLimitParameters(session, parameters);
         addShowPrivateParameter(parameters, session);
-
-        JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), CommonConstants.ACTION_ALL, session, parameters);
-        JSONArray jArray = extractArrayResult(jsonResult);
         List<DataObject> resultList = new ArrayList<DataObject>();
-        for (int i = 0; i < jArray.length(); i++) {
-            JSONArray columnsArray;
-            DataObject destination = null;
+        
+        long nParts = 1L;
+        long currentPart = 0L;
+        
+        while (currentPart < nParts) {
+            if (nParts == 1L) {
+                addTimeLimitParameters(parameters, session);
+            }
+            else {
+                long width = ( session.getCalendarEndDate() - session.getCalendarStartDate() ) / nParts;
+                parameters.put(CommonConstants.START, String.valueOf(session.getCalendarStartDate() + currentPart*width));
+                parameters.put(CommonConstants.END, String.valueOf(session.getCalendarStartDate()+ (currentPart+1L)*width));
+            }
+    
             try {
-                columnsArray = jArray.getJSONArray(i);
-                destination = _contentType.newDataObject(session);
-                updateDataObjectFromJSONArray(destination, requestedFields, columnsArray);
-                readOptionalTimestamp(jsonResult.getJSONObject(), destination);
-                if (seriesObject != null) {
-                    if (destination.getID().equals(seriesObject.getID()) || seriesObject.getID().equals(
-                        String.valueOf(destination.getFieldContent(CalendarConstants.RECURRENCE_ID))))
-                        resultList.add(destination);
-                } else {
-                    resultList.add(destination);
+                JSONResult jsonResult = _ajaxAccess.doGet(getOXAjaxAccessPath(), CommonConstants.ACTION_ALL, session, parameters);
+                JSONArray jArray = extractArrayResult(jsonResult);
+                for (int i = 0; i < jArray.length(); i++) {
+                    JSONArray columnsArray;
+                    DataObject destination = null;
+                    try {
+                        columnsArray = jArray.getJSONArray(i);
+                        destination = _contentType.newDataObject(session);
+                        updateDataObjectFromJSONArray(destination, requestedFields, columnsArray);
+                        readOptionalTimestamp(jsonResult.getJSONObject(), destination);
+                        if (seriesObject != null) {
+                            if (destination.getID().equals(seriesObject.getID()) || seriesObject.getID().equals(
+                                String.valueOf(destination.getFieldContent(CalendarConstants.RECURRENCE_ID))))
+                                resultList.add(destination);
+                        } else {
+                            resultList.add(destination);
+                        }
+                    } catch (JSONException e) {
+                        throw generateException(
+                            USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER1,
+                            destination,
+                            "get new appointmets:data not present",
+                            e);
+                    }
+                }
+                currentPart++;
+            }
+            catch (OXCommunicationException oxe) {
+                try {
+                    if (nParts < 100L && "CAL-5072".equals(oxe.getOxErrorForJSONResponse().get("code"))) {
+                        nParts++;
+                        currentPart = 0L;
+                        resultList.clear();
+                    }
+                    else {
+                        // giving up
+                        throw(oxe);
+                    }
+                } catch (JSONException e) {
+                    throw(oxe);
                 }
-            } catch (JSONException e) {
-                throw generateException(
-                    USMContentTypesUtilErrorCodes.DATA_NOT_PRESENT_NUMBER1,
-                    destination,
-                    "get new appointmets:data not present",
-                    e);
             }
         }
         DataObject[] result = resultList.toArray(new DataObject[resultList.size()]);
@@ -356,37 +389,100 @@ public class AppointmentContentTypeTransferHandler extends AbstractTransferHandl
         Map<String, String> parameters = new HashMap<String, String>();
         JSONObject requestBody = createRequestBodyWithIgnoreConflicts(object, false);
         // printTimeValues("New", requestBody);
-        JSONResult result = _ajaxAccess.doPut(
-            CalendarConstants.CALENDAR_PATH,
-            CommonConstants.ACTION_NEW,
-            object.getSession(),
-            parameters,
-            requestBody);
-        checkResult(result, JSONResultType.JSONObject);
-        JSONObject o = result.getJSONObject();
-        readOptionalTimestamp(o, object);
-        try {
-            JSONObject resultObject = o.getJSONObject(CommonConstants.RESULT_DATA);
-            if (resultObject.has(CommonConstants.RESULT_ID)) {
-                object.setID(resultObject.getString(CommonConstants.RESULT_ID));
-            } else if (resultObject.has(CalendarConstants.CONFLICTS)) {
-                JSONArray arr = resultObject.getJSONArray(CalendarConstants.CONFLICTS);
+        int retryCount = 50;
+        String previousInvalidChars = "";
+        while (retryCount-- > 0) {
+            JSONResult result = _ajaxAccess.doPut(
+                CalendarConstants.CALENDAR_PATH,
+                CommonConstants.ACTION_NEW,
+                object.getSession(),
+                parameters,
+                requestBody);
+            JSONArray errorParams = checkInvalidCharacters(result.getJSONObject());
+            if (errorParams == null)
+                checkResult(result, JSONResultType.JSONObject);
+
+            JSONObject o = result.getJSONObject();
+            readOptionalTimestamp(o, object);
+            try {
+                if (errorParams != null) {
+                    if (retryCount == 0) {
+                        LOG.warn(o.optString(CommonConstants.ERROR));
+                    }
+                    String contentKey = errorParams.getString(0);
+                    String invalidChars = errorParams.getString(1);
+                    if (invalidChars == null || invalidChars.equals(previousInvalidChars)) {
+                        throw new OXCommunicationException(
+                            USMContentTypesCalendarErrorCodes.OTHER_ERROR,
+                            "Appointment could not be created because of invalid content",
+                            o.toString());
+                    }
+                    previousInvalidChars = invalidChars;
+                    
+                    byte[] bytes = invalidChars.getBytes(Charset.forName("UTF-8"));
+                    String HEX_DIGITS = "0123456789ABCDEF";
+                    StringBuilder sb = new StringBuilder(bytes.length + 3);
+                    for (byte bb: bytes) {
+                        int b = bb & 0xFF;
+                        sb.append(HEX_DIGITS.charAt(b >> 4)).append(HEX_DIGITS.charAt(b % 16)).append(' ');
+                    }
+                    LOG.debug("invalid characters as bytes: " + sb.toString());
+                    
+                    if (!requestBody.has(contentKey)) {
+                        contentKey = contentKey.toLowerCase();
+                    }
+                    if (!requestBody.has(contentKey)) {
+                        throw new OXCommunicationException(
+                            USMContentTypesCalendarErrorCodes.OTHER_ERROR,
+                            "Appointment could not be created because of invalid content",
+                            o.toString());
+                    }
+                    String replacedContent = requestBody.getString(contentKey);
+
+                    int offset = 0, strLen = invalidChars.length();
+                    StringBuilder sb3 = new StringBuilder(strLen + 3);
+                    while (offset < strLen) {
+                      int curChar = invalidChars.codePointAt(offset);
+                      int charLen = Character.charCount(curChar);
+                      sb3.append(invalidChars.substring(offset, offset+charLen));
+                      sb3.append(": ");
+                      while (curChar > 0) {
+                          int b = curChar & 0xFF;
+                          sb3.append(HEX_DIGITS.charAt(b >> 4)).append(HEX_DIGITS.charAt(b % 16)).append(' ');
+                          curChar = curChar >> 8;
+                      }
+                      sb3.append(";  ");
+                      replacedContent = replacedContent.replaceAll(invalidChars.substring(offset, offset+charLen), "");
+                      offset += charLen;
+                    }
+                    LOG.debug("codePoints: " +  sb3.toString());
+                    requestBody.put(contentKey, replacedContent);
+                    object.setFieldContent(contentKey, replacedContent);
+                } else {
+                    JSONObject resultObject = o.getJSONObject(CommonConstants.RESULT_DATA);
+                    if (resultObject.has(CommonConstants.RESULT_ID)) {
+                        object.setID(resultObject.getString(CommonConstants.RESULT_ID));
+                        retryCount = 0;
+                    } else if (resultObject.has(CalendarConstants.CONFLICTS)) {
+                        JSONArray arr = resultObject.getJSONArray(CalendarConstants.CONFLICTS);
+                        throw new OXCommunicationException(
+                            USMContentTypesCalendarErrorCodes.CONFLICTS,
+                            "Appointment could not be created because of conflicts",
+                            arr.getJSONObject(0));
+                    } else {
+                        throw new OXCommunicationException(
+                            USMContentTypesCalendarErrorCodes.OTHER_ERROR,
+                            "Appointment could not be created because of ox communication error",
+                            resultObject.toString());
+                    }
+                }
+            } catch (JSONException e) {
                 throw new OXCommunicationException(
-                    USMContentTypesCalendarErrorCodes.CONFLICTS,
-                    "Appointment could not be created because of conflicts",
-                    arr.getJSONObject(0));
-            } else {
-                throw new OXCommunicationException(
-                    USMContentTypesCalendarErrorCodes.OTHER_ERROR,
-                    "Appointment could not be created because of ox communication error",
-                    resultObject.toString());
+                    USMContentTypesUtilErrorCodes.OX_RESULT_ERROR_NUMBER1,
+                    "Invalid Result on new " + DefaultContentTypes.CALENDAR_ID,
+                    e,
+                    result.toString());
             }
-        } catch (JSONException e) {
-            throw new OXCommunicationException(
-                USMContentTypesUtilErrorCodes.OX_RESULT_ERROR_NUMBER1,
-                "Invalid Result on new " + DefaultContentTypes.CALENDAR_ID,
-                e,
-                result.toString());
         }
         writeDeletedExceptions(object);
     }
