@@ -70,25 +70,23 @@ public class ChangeCollationUpdateTask extends USMBaseUpdateTaskV2 {
 
 	@Override
 	protected void doPerform(PerformParameters params) throws OXException {
-		int contextId = params.getContextId();
-		Connection con = null;
+		Connection con = params.getConnection();
 		Statement stmt = null;
 		try {
-			con = _provider.getDatabaseService().getForUpdateTask(contextId);
 			con.setAutoCommit(false);
 			stmt = con.createStatement();
-			for (String table : TableDefinition.TABLE_NAMES)
-				alterCollationForTable(table, stmt);
-			for (String table : USMUUIDTablesUpdateTaskV2.TABLE_NAMES)
-				alterCollationForTable(table, stmt);
+			for (String table : TableDefinition.TABLE_NAMES) {
+                alterCollationForTable(table, stmt);
+            }
+			for (String table : USMUUIDTablesUpdateTaskV2.TABLE_NAMES) {
+                alterCollationForTable(table, stmt);
+            }
 			con.commit();
 		} catch (SQLException e) {
 			rollback(con);
 			throw USMUpdateExceptionCodes.SQL_ERROR.create(e);
 		} finally {
 			enableAutocommit(con, stmt);
-			if (con != null)
-				_provider.getDatabaseService().backWritable(contextId, con);
 		}
 	}
 
