@@ -178,6 +178,9 @@ public class SyncUpdateHandler extends SyncCommandHandler {
                 // and replace the uuids created on the server with the original uuids sent from the client
                 retrieveExceptionsFromServerAndSetUUIDs(syncId, _folderToSync);
             }
+            if (_syncResult.getChanges() != null && _syncResult.getChanges().length > 0) {
+                _newStateToSave = new DataObjectSet(_syncResult.getNewState());
+            }
             syncId = finishProcessingAndSaveNewState(_syncResult, _folderToSync, syncId);
             JSONObject response = createResponseFromSyncResults(_syncResult, syncId);
             _swapUUIDsMap.clear();
