@@ -52,14 +52,12 @@ package com.openexchange.usm.ox_json.impl;
 import static com.openexchange.usm.util.OXJSONUtil.DATA_FIELD;
 import static com.openexchange.usm.util.OXJSONUtil.ERROR_FIELD;
 import static com.openexchange.usm.util.OXJSONUtil.isOXError;
-
+import java.io.BufferedReader;
 import java.io.IOException;
-
-import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-import org.json.JSONTokener;
-
+import org.json.JSONValue;
+import com.openexchange.java.Streams;
 import com.openexchange.usm.api.ox.json.JSONResult;
 import com.openexchange.usm.api.ox.json.JSONResultType;
 import com.openexchange.usm.util.OXJSONUtil;
@@ -67,19 +65,18 @@ import com.openexchange.usm.util.OXJSONUtil;
 public class OXJSONSupport {
 
 
-    public static JSONResult parseResult(String content, JSONResultType expectedResultType) throws IOException {
-        JSONTokener tokener = new JSONTokener(content);
+    public static JSONResult parseResult(BufferedReader contentReader, JSONResultType expectedResultType) throws IOException {
+        BufferedReader reader = contentReader;
         try {
-            char c = tokener.nextClean();
-            switch (c) {
-            case '[':
-            case '(':
-                tokener.back();
-                JSONArrayResult result = new JSONArrayResult(new JSONArray(tokener));
-                if (tokener.nextClean() == 0) {
-                    if (isValidResultType(expectedResultType, JSONResultType.JSONArray))
+            reader.mark(8192);
+            try {
+                JSONValue jValue = JSONObject.parse(reader);
+                if (jValue.isArray()) {
+                    JSONArrayResult result = new JSONArrayResult(jValue.toArray());
+                    if (isValidResultType(expectedResultType, JSONResultType.JSONArray)) {
                         return result;
-                    JSONObject errorDetails = new JSONObject();
+                    }
+                    JSONObject errorDetails = new JSONObject(4);
                     try {
                         errorDetails.put(DATA_FIELD, result.getJSONArray());
                         errorDetails.put(ERROR_FIELD, "Invalid array response from server, expected " + expectedResultType);
@@ -88,44 +85,58 @@ public class OXJSONSupport {
                     }
                     return new JSONErrorResult(errorDetails);
                 }
-                break;
-            case '{':
-                tokener.back();
-                JSONObject object = new JSONObject(tokener);
-                if (tokener.nextClean() == 0)
-                    return generateJSONResultForJSONObject(object, expectedResultType);
-                break;
-            default:
-                break;
+
+                // Otherwise a JSON object...
+                JSONObject object = jValue.toObject();
+                JSONResult jsonResult = generateJSONResultForJSONObject(object, expectedResultType);
+                Streams.close(reader);
+                reader = null;
+                return jsonResult;
+            } catch (JSONException e) {
+                // Neither a JSON object or a JSON array
             }
-        } catch (JSONException ignored) {
-            //Should not happen - object creation
+            reader.reset();
+
+            StringBuilder sb = new StringBuilder(2048);
+            char[] cbuf = new char[8192];
+            for (int read; (read = reader.read(cbuf, 0, cbuf.length)) > 0;) {
+                sb.append(cbuf, 0, read);
+            }
+            Streams.close(reader);
+            reader = null;
+
+            if (isValidResultType(expectedResultType, JSONResultType.String)) {
+                return new JSONStringResult(sb.toString());
+            }
+
+            JSONObject errorDetails = new JSONObject(4);
+            try {
+                errorDetails.put(DATA_FIELD, sb.toString());
+                errorDetails.put(ERROR_FIELD, "Invalid string response from server, expected " + expectedResultType);
+            } catch (JSONException ignored) {
+              //Should not happen - object creation
+            }
+            return new JSONErrorResult(errorDetails);
+        } finally {
+            Streams.close(reader);
         }
-        if (isValidResultType(expectedResultType, JSONResultType.String))
-            return new JSONStringResult(content);
-        JSONObject errorDetails = new JSONObject();
-        try {
-            errorDetails.put(DATA_FIELD, content);
-            errorDetails.put(ERROR_FIELD, "Invalid string response from server, expected " + expectedResultType);
-        } catch (JSONException ignored) {
-          //Should not happen - object creation
-        }
-        return new JSONErrorResult(errorDetails);
     }
 
     /**
      * if JSONObject has a field 'error' and the category of this error is not "13" then return JSONErrorResult if a field 'data' is
      * available and is not empty then observe the content of this field; if starts with '[' or '(' return JSONArrayResult if starts with
      * '{' return JSONObjectResult
-     * 
+     *
      * @param object
      * @return
      */
     public static JSONResult generateJSONResultForJSONObject(JSONObject object, JSONResultType expectedResultType) {
-        if (isOXError(object))
+        if (isOXError(object)) {
             return new JSONErrorResult(object);
-        if (isValidResultType(expectedResultType, JSONResultType.JSONObject))
+        }
+        if (isValidResultType(expectedResultType, JSONResultType.JSONObject)) {
             return new JSONObjectResult(object);
+        }
         JSONObject errorDetails = new JSONObject();
         try {
             errorDetails.put(DATA_FIELD, object);
@@ -136,7 +147,7 @@ public class OXJSONSupport {
         return new JSONErrorResult(errorDetails);
     }
 
-   
+
 
     private static boolean isValidResultType(JSONResultType expected, JSONResultType actual) {
         return expected == null || expected == actual;
